import datetime as dt
import pandas as pd
from getpass import getpass
from report_generator.html_tools import banner, color_map, data_url
from report_generator.smtp import sendHTMLmailwithattachments, set_credentials, set_password

config_map = {
    'team1': ('Team Name','Sales & Marketing','blue')
}


class Report:
    def __init__(self, team, title):
        assert team in config_map.keys(), f"{team} is not found in config_map"
        self.team = team
        self.title = title
        self.widgets = []
        self.attachments = []

    def add(self,widget,header=""):
        if type(widget) == pd.DataFrame:
            html = widget.to_html(classes='report')
        else:
            html = f"{widget}<br>"
        if header != "":
            html = f'<h2>{header}</h2>{html}'
        self.widgets.append(html)

    def add_image(self,file_name,header=""):
        name = 'attachment' + str(len(self.attachments))
        self.attachments.append({
            'name': name,
            'filename': file_name
        })
        html = f"<img src='cid:{name}'>"
#        html = f"<img src='{data_url(file_name)}'>"
        self.add(html, header)


    def to_html(self):
        date = dt.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        config = config_map[self.team]
        body = "\n".join(self.widgets)
        color = color_map()[config[2]]
        html = banner(date,self.title,config[0],config[1],'',color,body=body)
        return html

    def save(self):
        html = self.to_html()
        with open('./report_generator/test.html','w') as f:
            f.write(html)
    
    def send(self, sender, to):
        body = self.to_html()
        subject = self.title
        sendHTMLmailwithattachments(sender, to, subject, body, attachments=self.attachments)

if __name__ == "__main__":
    report = Report('team1',"Fancy report")
    report.add("This is a test","Header of this section")
    report.add("This is another test", "Header of another section")
    report.add_image('./report_generator/spmini.png', "Sample image")
    report.add(pd.read_csv('http://bit.ly/drinksbycountry').head(15), "Sample dataframe")
    sender = 'sender@gmail.com'
    #set_password()
    set_credentials('smtp.gmail.com',sender,sender)
    #report.send(sender,[sender])
    report.save()


import base64

def get_css(color):
    font = '"Calibri", Arial, Helvetica, sans-serif' 
    return f"""
    body                         {{ font-family: "Calibri"; font-size: 10pt; padding: 2px 7px; }}
    p, h2, h3, h4, h5, h6, TD    {{ {font} }}
    h1                           {{ {font}; color:#{color}; border-bottom: solid 1px }}
    .dateheader {{ padding-left: 1pt; font-size:14pt }}
    .underline  {{ border-bottom: solid 1px; margin-bottom:20pt; padding-bottom: 5pt; color: #{color} }}
    .report thead tr {{ background-color: #{color}; color: white }}
    .report {{ border-collapse: collapse; margin: 10px; }}
    .banner{color}     {{ 
        $FONT; 
        background-color: #{color}; 
	color: #ffffff; 
	font-size: 16pt; 
	padding: 5pt 10pt; 
	text-align: left; 
	background: linear-gradient(45deg, #{color}, #{color}55); 
    }}
    
    """


def banner(date, title, banner_title, banner_subtitle, imagelink, color, body = ""):
    style = get_css(color)
    return f"""
    <html>
<Title>{title}</Title>
        <head>
            <style>{style}</style>
        </head>
        <body>

<table style='width: 100%'><tr>
	<td style='width:85px'><img align=left src='{imagelink}' height=70></td>
	<td  class=banner{color}><B>{banner_title}</B><BR><font-size=-1>{banner_subtitle}</font></td>
	</tr>
</table>
<br>		
<a class=dateheader>{date}</a><BR>
<div class='dateheader underline'><b>{title}</b></div>
{body}
</body>
</html>
"""

def encode_image(filename):
    encoded_string = ""
    with open(filename, "rb") as image_file:
        encoded_string = base64.b64encode(image_file.read())
        file = encoded_string
    return file.decode('ascii')

def data_url(filename):
    return f'data:image/gif;base64,{encode_image(filename)}'

def color_map():
    return \
    {
        'blue': '125AAA',
    #    'blanchedalmond': 'FFEBCD',
        'brown': 'A52A2A',
        'cadetblue': '5F9EA0',
        'chocolate': 'D2691E',
    #    'cornsilk': 'FFF8DC',
        'crimson': 'DC143C',
        'darkblue': '00008B',
        'darkmagenta': '8B008B',
        'darkorchid': '9932CC',
        'darkgreen': '006400',
        'darkslateblue': '483D8B',
        'darkslategray': '2F4F4F',
        'firebrick': 'B22222',
        'indigo': '4B0082',
        'orangered': 'FF4500',
        'teal': '008080',
        'gold': 'FFD700'
    }


if __name__ == "__main__":
    date,title,banner_title, banner_subtitle = 'Monday, July 6th 2020', 'Fancy sales report', 'Team Name', 'Sales & Marketing'
    imagelink, color = '', '125aaa'

    colors = color_map()
    with open(f'output/test-all.html','w') as f:
       for name,color in colors.items():
          print(color, name)
          html = banner(date,title,banner_title , f"{banner_subtitle} - theme = {name}",  imagelink,  color)
          f.write(html)
        

    for name,color in colors.items():
       with open(f'output/test-{name}.html','w') as f:
          print(color, name)
          html = banner(date,title,banner_title, banner_subtitle,  imagelink,  color)
          f.write(html)



import setuptools
from subprocess import check_output

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as fh:
    requirements = fh.read().split("\n")
    requirements = [r for r in requirements if r != "" ]

def git_tag():
    tag = check_output(['git','describe','--tags'])
    tag = tag.decode('ascii')[:-1]
    print("Getting git tag: " + tag)
    return tag

setuptools.setup(
    name="report-generator", # Replace with your own username
    version=git_tag()[1:],
    author="Joel Horowitz",
    author_email="joelhoro@gmail.com",
    description="HTML report generator - note that github repo is private",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/joelhoro/report-generator",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=requirements,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



#################################################################
### Title     :     SMTP
### Filename  :     smtp.py
### Created   :     2012
### Author    :     Joel Horowitz
### Type      :     Library
### Summary   :
###
###
#################################################################

import smtplib, os
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

_server, _email, _login = None, None, None
def set_credentials(server,email,login):
    global _login, _server, _email
    _login = login
    _server = server
    _email = email
    print(_server,_email,_login,'********')

def set_password(pw):
    os.environ['SMTP_PWD'] = pw

def get_password():
    return os.environ['SMTP_PWD']


def sendHTMLmailwithattachments(sender,to,subject,body,attachments = [], cc = []):
    COMMASPACE = ', '
    
    # Create the container (outer) email message.
    msg = MIMEMultipart('related')
    msg['Subject'] = subject
    msg['From'] = _email
    if type(to)==str:
        to = [to]
    msg['To'] = COMMASPACE.join(to)
    
    msgAlternative = MIMEMultipart('alternative')
    msg.attach(msgAlternative)
    
    msgText = MIMEText(body, 'html')
    msgAlternative.attach(msgText)
    
    for FileInfo in attachments:
        fp = open(FileInfo['filename'], 'rb')
        msgImage = MIMEImage(fp.read())
        fp.close()
        msgImage.add_header('Content-ID', '<%s>' % FileInfo['name'])
        msg.attach(msgImage)
    
    try:
        with smtplib.SMTP(_server, 587) as server:
            server.starttls()
            print(f"Logging into {_server} as {_login}...")
            server.login(_login,get_password())
            print(f"EHLO...", end="")
            server.ehlo()
            print(f"Sending mail...", end="")
            server.sendmail(sender, to, msg.as_string())
            print(f"Closing connection")
            server.quit()
        print(f"Email sent to {to}: {subject}")
    except Exception as e:
        print(f"An error occurred: {e}")
if __name__ == "__main__":
    email = 'user@domain.com'
    pw = 'blah'
    set_credentials('smtp.gmail.com', email, email, pw)    
    sendHTMLmailwithattachments(email,[recipient],'Test','Body')
